#include <stdlib.h>
#include <map>
#include <string>
#include "FirePhish.h"
#include <phish.h>
#include <nsMemory.h>
#include <prthread.h>

using std::map;
using std::string;

NS_IMPL_ISUPPORTS1(FirePhish, IFirePhish)

/* void InitPhish (in string agent, in string cver); */
NS_IMETHODIMP FirePhish::InitPhish(const char *agent, const char *cver)
{
  phish_init(agent, cver);
  UpdateXML();
  return NS_OK;
}


FirePhish::FirePhish()
{
  
  //Allocate latest results object
  latest_results = (phish_url_data_t*)nsMemory::Alloc(sizeof(phish_url_data_t));
  
  latest_checked_site = NULL;

  //Initialize the phish_result object
  latest_results->comments = NULL;
  ResetResults();

  m_sl_entry = NULL;
  
  //Initialize TLD-to-fullCountryName map
  //Information taken from http://www.iana.org/ (official site of the
  //Internet Assigned Numbers Authority).
  tld_to_name["AC"] = "Ascension Island";
  tld_to_name["AE"] = "United Arab Emirates";
  tld_to_name["AF"] = "Afghanistan";
  tld_to_name["AG"] = "Antigua and Barbuda";
  tld_to_name["AI"] = "Anguilla";
  tld_to_name["AL"] = "Albania";
  tld_to_name["AM"] = "Armenia";
  tld_to_name["AN"] = "Netherlands Antilles";
  tld_to_name["AO"] = "Angola";
  tld_to_name["AQ"] = "Antarctica";
  tld_to_name["AR"] = "Argentina";
  tld_to_name["AS"] = "American Samoa";
  tld_to_name["AT"] = "Austria";
  tld_to_name["AU"] = "Australia";
  tld_to_name["AW"] = "Aruba";
  tld_to_name["AZ"] = "Azerbaijan";
  tld_to_name["AX"] = "Aland Islands";
  tld_to_name["BA"] = "Bosnia and Herzegovina";
  tld_to_name["BB"] = "Barbados";
  tld_to_name["BD"] = "Bangladesh";
  tld_to_name["BE"] = "Belgium";
  tld_to_name["BF"] = "Burkina Faso";
  tld_to_name["BG"] = "Bulgaria";
  tld_to_name["BH"] = "Bahrain";
  tld_to_name["BI"] = "Burundi";
  tld_to_name["BJ"] = "Benin";
  tld_to_name["BM"] = "Bermuda";
  tld_to_name["BN"] = "Brunei Darussalam";
  tld_to_name["BO"] = "Bolivia";
  tld_to_name["BR"] = "Brazil";
  tld_to_name["BS"] = "Bahamas";
  tld_to_name["BT"] = "Bhutan";
  tld_to_name["BV"] = "Bouvet Island";
  tld_to_name["BW"] = "Botswana";
  tld_to_name["BY"] = "Belarus";
  tld_to_name["BZ"] = "Belize";
  tld_to_name["CA"] = "Canada";
  tld_to_name["CC"] = "Cocos (Keeling) Islands";
  tld_to_name["CD"] = "Congo, The Democratic Republic of the";
  tld_to_name["CF"] = "Central African Republic";
  tld_to_name["CG"] = "Congo, Republic of";
  tld_to_name["CH"] = "Switzerland";
  tld_to_name["CI"] = "Cote d'Ivoire";
  tld_to_name["CK"] = "Cook Islands";
  tld_to_name["CL"] = "Chile";
  tld_to_name["CM"] = "Cameroon";
  tld_to_name["CN"] = "China";
  tld_to_name["CO"] = "Colombia";
  tld_to_name["CR"] = "Costa Rica";
  tld_to_name["CS"] = "Serbia and Montenegro";
  tld_to_name["CU"] = "Cuba";
  tld_to_name["CV"] = "Cape Verde";
  tld_to_name["CX"] = "Christmas Island";
  tld_to_name["CY"] = "Cyprus";
  tld_to_name["CZ"] = "Czech Republic";
  tld_to_name["DE"] = "Germany";
  tld_to_name["DJ"] = "Djibouti";
  tld_to_name["DK"] = "Denmark";
  tld_to_name["DM"] = "Dominica";
  tld_to_name["DO"] = "Dominican Republic";
  tld_to_name["DZ"] = "Algeria";
  tld_to_name["EC"] = "Ecuador";
  tld_to_name["EE"] = "Estonia";
  tld_to_name["EG"] = "Egypt";
  tld_to_name["EH"] = "Western Sahara";
  tld_to_name["ER"] = "Eritrea";
  tld_to_name["ES"] = "Spain";
  tld_to_name["ET"] = "Ethiopia";
  tld_to_name["EU"] = "European Union";
  tld_to_name["FI"] = "Finland";
  tld_to_name["FJ"] = "Fiji";
  tld_to_name["FK"] = "Falkland Islands (Malvinas)";
  tld_to_name["FM"] = "Micronesia, Federal State of";
  tld_to_name["FO"] = "Faroe Islands";
  tld_to_name["FR"] = "France";
  tld_to_name["GA"] = "Gabon";
  tld_to_name["GB"] = "United Kingdom";
  tld_to_name["GD"] = "Grenada";
  tld_to_name["GE"] = "Georgia";
  tld_to_name["GF"] = "French Guiana";
  tld_to_name["GG"] = "Guernsey";
  tld_to_name["GH"] = "Ghana";
  tld_to_name["GI"] = "Gibraltar";
  tld_to_name["GL"] = "Greenland";
  tld_to_name["GM"] = "Gambia";
  tld_to_name["GN"] = "Guinea";
  tld_to_name["GP"] = "Guadeloupe";
  tld_to_name["GQ"] = "Equatorial Guinea";
  tld_to_name["GR"] = "Greece";
  tld_to_name["GS"] = "South Georgia and the South Sandwich Islands";
  tld_to_name["GT"] = "Guatemala";
  tld_to_name["GU"] = "Guam";
  tld_to_name["GW"] = "Guinea-Bissau";
  tld_to_name["GY"] = "Guyana";
  tld_to_name["HK"] = "Hong Kong";
  tld_to_name["HM"] = "Heard and McDonald Islands";
  tld_to_name["HN"] = "Honduras";
  tld_to_name["HR"] = "Croatia/Hrvatska";
  tld_to_name["HT"] = "Haiti";
  tld_to_name["HU"] = "Hungary";
  tld_to_name["ID"] = "Indonesia";
  tld_to_name["IE"] = "Ireland";
  tld_to_name["IL"] = "Israel";
  tld_to_name["IM"] = "Isle of Man";
  tld_to_name["IN"] = "India";
  tld_to_name["IO"] = "British Indian Ocean Territory";
  tld_to_name["IQ"] = "Iraq";
  tld_to_name["IR"] = "Iran, Islamic Republic of";
  tld_to_name["IS"] = "Iceland";
  tld_to_name["IT"] = "Italy";
  tld_to_name["JE"] = "Jersey";
  tld_to_name["JM"] = "Jamaica";
  tld_to_name["JO"] = "Jordan";
  tld_to_name["JP"] = "Japan";
  tld_to_name["KE"] = "Kenya";
  tld_to_name["KG"] = "Kyrgyzstan";
  tld_to_name["KH"] = "Cambodia";
  tld_to_name["KI"] = "Kiribati";
  tld_to_name["KM"] = "Comoros";
  tld_to_name["KN"] = "Saint Kitts and Nevis";
  tld_to_name["KP"] = "Korea, Democratic People's Republic";
  tld_to_name["KR"] = "Korea, Republic of";
  tld_to_name["KW"] = "Kuwait";
  tld_to_name["KY"] = "Cayman Islands";
  tld_to_name["KZ"] = "Kazakhstan";
  tld_to_name["LA"] = "Lao People's Democratic Republic";
  tld_to_name["LB"] = "Lebanon";
  tld_to_name["LC"] = "Saint Lucia";
  tld_to_name["LI"] = "Liechtenstein";
  tld_to_name["LK"] = "Sri Lanka";
  tld_to_name["LR"] = "Liberia";
  tld_to_name["LS"] = "Lesotho";
  tld_to_name["LT"] = "Lithuania";
  tld_to_name["LU"] = "Luxembourg";
  tld_to_name["LV"] = "Latvia";
  tld_to_name["LY"] = "Libyan Arab Jamahiriya";
  tld_to_name["MA"] = "Morocco";
  tld_to_name["MC"] = "Monaco";
  tld_to_name["MD"] = "Moldova, Republic of";
  tld_to_name["MG"] = "Madagascar";
  tld_to_name["MH"] = "Marshall Islands";
  tld_to_name["MK"] = "Macedonia, The Former Yugoslav Republic of";
  tld_to_name["ML"] = "Mali";
  tld_to_name["MM"] = "Myanmar";
  tld_to_name["MN"] = "Mongolia";
  tld_to_name["MO"] = "Macau";
  tld_to_name["MP"] = "Northern Mariana Islands";
  tld_to_name["MQ"] = "Martinique";
  tld_to_name["MR"] = "Mauritania";
  tld_to_name["MS"] = "Montserrat";
  tld_to_name["MT"] = "Malta";
  tld_to_name["MU"] = "Mauritius";
  tld_to_name["MV"] = "Maldives";
  tld_to_name["MW"] = "Malawi";
  tld_to_name["MX"] = "Mexico";
  tld_to_name["MY"] = "Malaysia";
  tld_to_name["MZ"] = "Mozambique";
  tld_to_name["NA"] = "Namibia";
  tld_to_name["NC"] = "New Caledonia";
  tld_to_name["NE"] = "Niger";
  tld_to_name["NF"] = "Norfolk Island";
  tld_to_name["NG"] = "Nigeria";
  tld_to_name["NI"] = "Nicaragua";
  tld_to_name["NL"] = "Netherlands";
  tld_to_name["NO"] = "Norway";
  tld_to_name["NP"] = "Nepal";
  tld_to_name["NR"] = "Nauru";
  tld_to_name["NU"] = "Niue";
  tld_to_name["NZ"] = "New Zealand";
  tld_to_name["OM"] = "Oman";
  tld_to_name["PA"] = "Panama";
  tld_to_name["PE"] = "Peru";
  tld_to_name["PF"] = "French Polynesia";
  tld_to_name["PG"] = "Papua New Guinea";
  tld_to_name["PH"] = "Philippines";
  tld_to_name["PK"] = "Pakistan";
  tld_to_name["PL"] = "Poland";
  tld_to_name["PM"] = "Saint Pierre and Miquelon";
  tld_to_name["PN"] = "Pitcairn Island";
  tld_to_name["PR"] = "Puerto Rico";
  tld_to_name["PS"] = "Palestinian Territories";
  tld_to_name["PT"] = "Portugal";
  tld_to_name["PW"] = "Palau";
  tld_to_name["PY"] = "Paraguay";
  tld_to_name["QA"] = "Qatar";
  tld_to_name["RE"] = "Reunion Island";
  tld_to_name["RO"] = "Romania";
  tld_to_name["RU"] = "Russian Federation";
  tld_to_name["RW"] = "Rwanda";
  tld_to_name["SA"] = "Saudi Arabia";
  tld_to_name["SB"] = "Solomon Islands";
  tld_to_name["SC"] = "Seychelles";
  tld_to_name["SD"] = "Sudan";
  tld_to_name["SE"] = "Sweden";
  tld_to_name["SG"] = "Singapore";
  tld_to_name["SH"] = "Saint Helena";
  tld_to_name["SI"] = "Slovenia";
  tld_to_name["SJ"] = "Svalbard and Jan Mayen Islands";
  tld_to_name["SK"] = "Slovak Republic";
  tld_to_name["SL"] = "Sierra Leone";
  tld_to_name["SM"] = "San Marino";
  tld_to_name["SN"] = "Senegal";
  tld_to_name["SO"] = "Somalia";
  tld_to_name["SR"] = "Suriname";
  tld_to_name["ST"] = "Sao Tome and Principe";
  tld_to_name["SV"] = "El Salvador";
  tld_to_name["SY"] = "Syrian Arab Republic";
  tld_to_name["SZ"] = "Swaziland";
  tld_to_name["TC"] = "Turks and Caicos Islands";
  tld_to_name["TD"] = "Chad";
  tld_to_name["TF"] = "French Southern Territories";
  tld_to_name["TG"] = "Togo";
  tld_to_name["TH"] = "Thailand";
  tld_to_name["TJ"] = "Tajikistan";
  tld_to_name["TK"] = "Tokelau";
  tld_to_name["TL"] = "Timor-Leste";
  tld_to_name["TM"] = "Turkmenistan";
  tld_to_name["TN"] = "Tunisia";
  tld_to_name["TO"] = "Tonga";
  tld_to_name["TP"] = "East Timor";
  tld_to_name["TR"] = "Turkey";
  tld_to_name["TT"] = "Trinidad and Tobago";
  tld_to_name["TV"] = "Tuvalu";
  tld_to_name["TW"] = "Taiwan";
  tld_to_name["TZ"] = "Tanzania";
  tld_to_name["UA"] = "Ukraine";
  tld_to_name["UG"] = "Uganda";
  tld_to_name["UK"] = "United Kingdom";
  tld_to_name["UM"] = "United States Minor Outlying Islands";
  tld_to_name["US"] = "United States";
  tld_to_name["UY"] = "Uruguay";
  tld_to_name["UZ"] = "Uzbekistan";
  tld_to_name["VA"] = "Holy See (Vatican City State)";
  tld_to_name["VC"] = "Saint Vincent and the Grenadines";
  tld_to_name["VE"] = "Venezuela";
  tld_to_name["VG"] = "Virgin Islands, British";
  tld_to_name["VI"] = "Virgin Islands, U.S.";
  tld_to_name["VN"] = "Vietnam";
  tld_to_name["VU"] = "Vanuatu";
  tld_to_name["WF"] = "Wallis and Futuna Islands";
  tld_to_name["WS"] = "Western Samoa";
  tld_to_name["YE"] = "Yemen";
  tld_to_name["YT"] = "Mayotte";
  tld_to_name["YU"] = "Yugoslavia";
  tld_to_name["ZA"] = "South Africa";
  tld_to_name["ZM"] = "Zambia";
  tld_to_name["ZW"] = "Zimbabwe";

}

FirePhish::~FirePhish()
{
  //Free the comments
  phish_deleteURLData(latest_results);

  //Release the latest results object
  nsMemory::Free(latest_results);

  phish_shutdown();
}


NS_IMETHODIMP FirePhish::CheckURL(const char *url_str, PRInt32 *_retval)
{
    phish_result_t res;

    latest_checked_site = (char*)nsMemory::Clone(url_str, strlen(url_str)+1);
    
    if ((res = phish_checkURL(url_str, latest_results)) != PHISH_SUCCESS)
    {
      *_retval = 0; //Unknown risk level
      return NS_OK;
    }

    //Return according to risk level
    switch (latest_results->risk_level)
    {
      case PHISH_RISK_UNKNOWN:
        *_retval = 0;
        return NS_OK;
      case PHISH_RISK_LOW:
        *_retval = 1;
        return NS_OK;
      case PHISH_RISK_MEDIUM:
        *_retval = 2;
        return NS_OK;
      case PHISH_RISK_HIGH:
        *_retval = 3;
        return NS_OK;    
      default:
        *_retval = 0;
        return NS_OK;
    }

    return NS_OK;
}

/* string GetCountry (); */
NS_IMETHODIMP FirePhish::GetCountry(char **_retval)
{
  string tldname(latest_results->country);
  string fullname = tld_to_name[tldname];
  const char *res = fullname.c_str();
  
  *_retval = (char*)nsMemory::Clone(res, strlen(res) + 1);
  return NS_OK;
}

/* string GetComments (); */
NS_IMETHODIMP FirePhish::GetComments(char **_retval)
{
  if (latest_results->comments)
    *_retval = (char*)nsMemory::Clone(latest_results->comments, strlen(latest_results->comments)+1);
  else
    *_retval = NULL;
  return NS_OK;
}

/* boolean CheckSafeList (in string url_str); */
NS_IMETHODIMP FirePhish::CheckSafeList(const char *url_str, PRBool *_retval)
{
  int reply = 0;
  phish_result_t res; 
  res = phish_checkSafeList(url_str, &reply);

  *_retval = (reply == 1);

  if (*_retval)
    latest_checked_site = (char*)nsMemory::Clone(url_str, strlen(url_str)+1);

  return NS_OK;
}



/* void SetRunningMode (in long mode); */
NS_IMETHODIMP FirePhish::SetRunningMode(PRInt32 mode)
{
  phish_mode_t md = (mode == 0) ? PHISH_OFFLINE_MODE : PHISH_ONLINE_MODE;
  phish_setRunningMode(md);

  return NS_OK;
}

/* void SetSiteQueryURL (in string url); */
NS_IMETHODIMP FirePhish::SetSiteQueryURL(const char *url)
{
  phish_setSiteQueryURL(url);
  return NS_OK;
}

/* void SetCountryQueryURL (in string url); */
NS_IMETHODIMP FirePhish::SetCountryQueryURL(const char *url)
{
  phish_setCountryQueryURL(url);
  return NS_OK;
}

/* void SetReportSiteURL (in string url); */
NS_IMETHODIMP FirePhish::SetReportSiteURL(const char *url)
{
  phish_setReportSiteURL(url);
  return NS_OK;
}

/* void SetRemoteXMLURL (in string url); */
NS_IMETHODIMP FirePhish::SetRemoteXMLURL(const char *url)
{
  phish_setRemoteXMLURL(url);
  return NS_OK;
}

/* void SetLocalXMLFile (in string path); */
NS_IMETHODIMP FirePhish::SetLocalXMLFile(const char *path)
{
  phish_setLocalXMLFile(path);
  return NS_OK;
}

/* void SetSafeListFile (in string path); */
NS_IMETHODIMP FirePhish::SetSafeListFile(const char *path)
{
  phish_setSafeListFile(path);
  return NS_OK;
}


/* long GetrunningMode (); */
NS_IMETHODIMP FirePhish::GetRunningMode(PRInt32 *_retval)
{
    phish_mode_t m;
    phish_runningMode(&m);
    
    int res = 0;
    if (m == PHISH_ONLINE_MODE)
      res = 1;

    *_retval = res;
    return NS_OK;
}

/* string GetsiteQueryURL (); */
NS_IMETHODIMP FirePhish::GetSiteQueryURL(char **_retval)
{
    const char* res;
    phish_siteQueryURL(&res);
    int len = strlen(res);

    *_retval = (char*)nsMemory::Clone(res, len + 1);
    return NS_OK;
}

/* string GetcountryQueryURL (); */
NS_IMETHODIMP FirePhish::GetCountryQueryURL(char **_retval)
{
    const char* res;
    phish_countryQueryURL(&res);
    int len = strlen(res);

    *_retval = (char*)nsMemory::Clone(res, len + 1);
    return NS_OK;
}

/* string GetreportSiteURL (); */
NS_IMETHODIMP FirePhish::GetReportSiteURL(char **_retval)
{
    const char* res;
    phish_reportSiteURL(&res);
    int len = strlen(res);

    *_retval = (char*)nsMemory::Clone(res, len + 1);
    return NS_OK;
}

/* string GetremoteXMLURL (); */
NS_IMETHODIMP FirePhish::GetRemoteXMLURL(char **_retval)
{
    const char* res;
    phish_remoteXMLURL(&res);
    int len = strlen(res);

    *_retval = (char*)nsMemory::Clone(res, len + 1);
    return NS_OK;
}

/* string GetlocalXMLFile (); */
NS_IMETHODIMP FirePhish::GetLocalXMLFile(char **_retval)
{
    const char* res;
    phish_localXMLFile(&res);
    int len = strlen(res);

    *_retval = (char*)nsMemory::Clone(res, len + 1);
    return NS_OK;
}

/* string GetsafeListFile (); */
NS_IMETHODIMP FirePhish::GetSafeListFile(char **_retval)
{
    const char* res;
    phish_safeListFile(&res);
    int len = strlen(res);

    *_retval = (char*)nsMemory::Clone(res, len + 1);
    return NS_OK;
}


/* string GetSafelistAt (in long i); */
NS_IMETHODIMP FirePhish::GetSafelistNext(char **_retval)
{
  const char *url;

  if (m_sl_entry == NULL)
    phish_getSafeListFirst(&m_sl_entry);
  else
    phish_getSafeListNext(m_sl_entry, &m_sl_entry);

  if (m_sl_entry == NULL)
    m_latest_safe_url[0] = '\0';
  else
  {
    phish_getSafeListData(m_sl_entry, &url);

    strcpy(m_latest_safe_url, url);
  }

  *_retval = (char*)nsMemory::Clone(m_latest_safe_url, strlen(m_latest_safe_url)+1);

  return NS_OK;
}


/* void CreateNewSafeList (); */
NS_IMETHODIMP FirePhish::CreateNewSafeList()
{
  phish_newSafeList(&m_safelist);
  return NS_OK;
}

/* void AddToSafeList (in string url); */
NS_IMETHODIMP FirePhish::AddToSafeList(const char *url)
{
  phish_addToSafeList(m_safelist, url);
  return NS_OK;
}

/* void AddToCurrentSafeList (in string url); */
NS_IMETHODIMP FirePhish::AddToCurrentSafeList(const char *url)
{
  phish_addToCurrentSafeList(url);
  return NS_OK;
}

/* void SetSafeList (); */
NS_IMETHODIMP FirePhish::SetSafeList()
{
  phish_setSafeList(m_safelist);
  phish_saveSafeList(); //Also save the safelist file.
  return NS_OK;
}


/* string GetReportingURL (in string url); */
NS_IMETHODIMP FirePhish::GetReportingURL(const char *url, char **_retval)
{
  char *temp;
  phish_getReportingURL(url, &temp);

  *_retval = (char*)nsMemory::Clone(temp, strlen(temp)+1);

  return NS_OK;
}


/* void UpdateXML (); */
NS_IMETHODIMP FirePhish::UpdateXML()
{
  PR_CreateThread(PR_USER_THREAD,
                  (void(*)(void*))phish_downloadDBAsXML,
                  NULL,
                  PR_PRIORITY_NORMAL,
                  PR_LOCAL_THREAD,
                  PR_UNJOINABLE_THREAD,
                  0);

  //phish_downloadDBAsXML();
  return NS_OK;
}


/* void ResetResults (); */
NS_IMETHODIMP FirePhish::ResetResults()
{
  
  //Free the comments
  phish_deleteURLData(latest_results);

  latest_results->comments = NULL;
  latest_results->comments_length = 0;
  latest_results->country[0] = '\0';
  latest_results->ip = -1;
  latest_results->path = -1;
  latest_results->server = -1;
  latest_results->risk_level = PHISH_RISK_UNKNOWN;
  latest_results->domain = -1;
  latest_results->suspicious_host = -1;
  latest_results->user_scheme = -1;

  nsMemory::Free(latest_checked_site);
  latest_checked_site = NULL;


  return NS_OK;
}


/* string GetLatestURL (); */
NS_IMETHODIMP FirePhish::GetLatestURL(char **_retval)
{
  if (latest_checked_site)
    *_retval = (char*)nsMemory::Clone(latest_checked_site, 
                strlen(latest_checked_site) + 1);
  else
    *_retval = NULL;
  return NS_OK;
}


/* long GetResultServer (); */
NS_IMETHODIMP FirePhish::GetResultServer(PRInt32 *_retval)
{
  *_retval = latest_results->server;
  return NS_OK;
}

/* long GetResultIP (); */
NS_IMETHODIMP FirePhish::GetResultIP(PRInt32 *_retval)
{
  *_retval = latest_results->ip;
  return NS_OK;
}

/* long GetResultPath (); */
NS_IMETHODIMP FirePhish::GetResultPath(PRInt32 *_retval)
{
  *_retval = latest_results->path;
  return NS_OK;
}

/* long GetResultDomain (); */
NS_IMETHODIMP FirePhish::GetResultDomain(PRInt32 *_retval)
{
  *_retval = latest_results->domain;
  return NS_OK;
}

/* long GetResultUserScheme (); */
NS_IMETHODIMP FirePhish::GetResultUserScheme(PRInt32 *_retval)
{
  *_retval = latest_results->user_scheme;
  return NS_OK;
}

/* long GetResultSuspiciousHost (); */
NS_IMETHODIMP FirePhish::GetResultSuspiciousHost(PRInt32 *_retval)
{
  *_retval = latest_results->suspicious_host;
  return NS_OK;
}

/* void CheckCountry (in string url); */
NS_IMETHODIMP FirePhish::CheckCountry(const char *url)
{
  phish_checkCountry(url, latest_results);
  return NS_OK;
}
